from snakypy.helpers import printer
from snakypy.helpers.ansi import FG
from snakypy.helpers.console import billboard, credence
from snakypy.zshpower.config import package
from snakypy.zshpower import __version__


class CreditsCommand:
    @staticmethod
    def run() -> None:
        print("\n")
        printer("Offered by:".center(50), foreground=FG().GREEN)
        billboard(
            package.info["organization_name"],
            justify="center",
            foreground=FG().YELLOW,
        )
        printer("copyright (c) since 2020\n".center(100), foreground=FG().GREEN)
        credence(
            package.info["name"],
            __version__,
            package.info["home_page"],
            package.info,
            foreground=FG().CYAN,
        )



import sqlite3
from typing import Any
from snakypy.zshpower.config.base import Base
from snakypy.zshpower import HOME
from snakypy.zshpower.database.sql import sql


class DAO(Base):
    def __init__(self):
        try:
            Base.__init__(self, HOME)
            self.conn = sqlite3.connect(self.database_path)
            self.get_cursor = self.conn.cursor()

        except sqlite3.Error:
            raise sqlite3.Error(
                "An error occurred while connecting to the database. Make sure that the SQLite database is turned on."
                "One way to resolve it is by running the command 'zshpower init [--omz]'\n>"
            )

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        self.commit()
        self.connection.close()

    @property
    def connection(self) -> sqlite3.Connection:
        return self.conn

    @property
    def cursor(self) -> sqlite3.Cursor:
        return self.get_cursor

    def commit(self):
        self.connection.commit()

    def execute(self, sql_, params=None):
        self.cursor.execute(sql_, params or ())

    def fetchall(self) -> list:
        return self.cursor.fetchall()

    def fetchone(self) -> Any:
        return self.cursor.fetchone()

    def query(self, sql_, params=None) -> list:
        self.cursor.execute(sql_, params or ())
        return self.fetchall()

    def query_one(self, sql_, params=None) -> Any:
        self.cursor.execute(sql_, params or ())
        return self.fetchone()

    def create_table(self, tbl_name) -> bool:
        try:
            self.execute(sql()[tbl_name])
            self.commit()
            self.connection.close()
            return True
        except (sqlite3.DatabaseError, sqlite3.DataError):
            return False

    def select_columns(self, /, columns=(), table=None) -> dict:
        sql_ = f"SELECT {','.join(columns)} FROM {table};"
        query = self.query(sql_)
        data = {key: value for (key, value) in query}
        self.connection.close()
        return data

    def select_where(self, table, value, where, select=()) -> list:
        sql_ = f"SELECT {','.join(select)} FROM {table} WHERE {where} = '{value}';"
        data = self.query(sql_)
        self.commit()
        self.connection.close()
        return data

    def insert(self, table, /, columns=(), values=()) -> bool:
        sql_ = f"INSERT INTO {table} {columns} VALUES {values};"
        try:
            self.execute(sql_)
            self.commit()
            self.connection.close()
            return True
        except Exception:
            raise Exception("Error insert database.")

    def update(self, table, set_, version, where, value) -> bool:
        sql_ = f"UPDATE {table} SET {set_} = '{version}' WHERE {where} = '{value}';"
        try:
            self.execute(sql_)
            self.commit()
            self.connection.close()
            return True
        except sqlite3.OperationalError as err:
            raise sqlite3.OperationalError(
                "Error writing to table. Table does not exist.", err
            )



from snakypy.zshpower.config.base import Base
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.catch import read_zshrc_omz
from snakypy.zshpower.utils.shift import change_theme_in_zshrc
from snakypy.zshpower.utils.process import reload_zsh
from snakypy.helpers.ansi import FG
from snakypy.helpers import printer


class ActivateCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self) -> bool:

        checking_init(self.HOME)

        if read_zshrc_omz(self.zsh_rc):
            if read_zshrc_omz(self.zsh_rc)[0] == "zshpower":
                printer("Already activated. Nothing to do.", foreground=FG().GREEN)
                exit(0)
            change_theme_in_zshrc(self.zsh_rc, "zshpower")
            printer("Activation process finish.", foreground=FG().FINISH)
            reload_zsh()
            return True
        printer(
            "You are not using Oh My ZSH to run this command.",
            foreground=FG().WARNING,
        )
        return False



from snakypy.helpers.catches import shell
from snakypy.helpers.ansi import FG, NONE
from snakypy.helpers.catches import whoami
from subprocess import call as subprocess_call


def reload_zsh() -> None:
    subprocess_call("exec zsh", shell=True)


def change_shell() -> bool:
    if shell()[0] != "zsh":
        try:
            subprocess_call(f"chsh -s $(which zsh) {whoami()}", shell=True)
            return True
        except KeyboardInterrupt as err:
            raise KeyboardInterrupt(f"{FG().WARNING}Canceled by user.{NONE}", err)
    return False



import curses
import sys
from sqlite3 import OperationalError
from snakypy.helpers import FG
from snakypy.helpers.console import loading, printer
from snakypy.zshpower.commands.lib.handle import records
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.config.base import Base


class Sync(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self) -> None:
        try:
            checking_init(self.HOME)
            records("update")
            curses.initscr()
            curses.curs_set(0)
            loading(
                set_time=0.040,
                bar=False,
                header="Synchronizing versions with database ...",
                foreground=FG().QUESTION,
            )
            curses.curs_set(1)
            curses.endwin()
            printer("Done!", foreground=FG().FINISH)
        except OperationalError:
            printer(
                "The database does not exist or is corrupted.",
                foreground=FG().ERROR,
            )
            sys.exit(1)



import curses
from snakypy.zshpower.database.dao import DAO
from snakypy.zshpower.utils.process import reload_zsh
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.shift import create_config
from snakypy.zshpower.config.config import content as config_content
from snakypy.helpers.console import loading
from snakypy.zshpower.config.base import Base
from snakypy.helpers.ansi import FG
from snakypy.helpers import printer
from snakypy.zshpower.commands.lib.handle import records


class ResetCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self, arguments) -> None:
        checking_init(self.HOME)
        if arguments["--config"]:
            create_config(config_content, self.config_file, force=True)
            printer("Reset process finished.", foreground=FG().FINISH)
            reload_zsh()
        elif arguments["--db"]:
            DAO().create_table(self.tbl_main)
            records("insert")
            curses.initscr()
            curses.curs_set(0)
            loading(
                set_time=0.050,
                bar=False,
                header="ZSHPower Restoring the database ...",
                foreground=FG().QUESTION,
            )
            curses.curs_set(1)
            curses.endwin()
            printer("Done!", foreground=FG().FINISH)



from snakypy.zshpower.config import package

# author = package.info["author"]["name"]
# website_author = package.info["author"]["website"]
# email = package.info["author"]["email"]

content = f"""#!/usr/bin/env zsh

## Information:
## ******************************************************************************
## Theme: {package.info["name"]}
## Description: Call script to start {package.info["name"]}.
## ******************************************************************************

## Export PATH local user
export PATH="$PATH:$HOME/.local/bin"

## Option using "add-zsh-hook"
function zshpower_precmd() {{
  state=$(which zshpower-draw > /dev/null 2>&1)
  if [ ! $? -ne 0 ]; then
    PROMPT="$(zshpower-draw prompt)"
    RPROMPT="$(zshpower-draw rprompt)"
  else
    PROMPT='%F{{green}}%n%f@%F{{magenta}}%m%f %F{{blue}}%B%~%b%f %# '
    RPROMPT='[%F{{yellow}}%?%f]'
  fi
}}

function install_zshpower_precmd() {{
  for s in "${{precmd_functions[@]}}"; do
    if [ "$s" = "zshpower_draw" ]; then
      return
    fi
  done
  precmd_functions+=(zshpower_precmd)
}}

## Option using "add-zsh-hook"
# autoload -Uz add-zsh-hook
# add-zsh-hook precmd zshpower_precmd

## Option not using "add-zsh-hook"
if [ "$TERM" != "linux" ]; then
   install_zshpower_precmd
fi
"""



from os import cpu_count
from snakypy.zshpower.prompt.sections.gulp import Gulp
from snakypy.zshpower.prompt.sections.zig import Zig
from snakypy.zshpower.prompt.sections.vagrant import Vagrant
from snakypy.zshpower.prompt.sections.ocaml import Ocaml
from snakypy.zshpower.prompt.sections.nim import Nim
from snakypy.zshpower.prompt.sections.kotlin import Kotlin
from snakypy.zshpower.prompt.sections.helm import Helm
from snakypy.zshpower.prompt.sections.erlang import Erlang
from snakypy.zshpower.prompt.sections.deno import Deno
from snakypy.zshpower.prompt.sections.crystal import Crystal
from snakypy.zshpower.prompt.sections.cmake import CMake
from snakypy.zshpower.prompt.sections.perl import Perl
from snakypy.zshpower.prompt.sections.julia import Julia
from snakypy.zshpower.prompt.sections.elixir import Elixir
from snakypy.zshpower.prompt.sections.dotnet import Dotnet
from snakypy.zshpower.prompt.sections.docker import Docker
from snakypy.zshpower.prompt.sections.golang import Golang
from snakypy.zshpower.prompt.sections.java import Java
from snakypy.zshpower.prompt.sections.scala import Scala
from snakypy.zshpower.prompt.sections.dart import Dart
from snakypy.zshpower.prompt.sections.nodejs import NodeJs
from snakypy.zshpower.prompt.sections.php import Php
from snakypy.zshpower.prompt.sections.ruby import Ruby
from snakypy.zshpower.prompt.sections.rust import Rust
from concurrent.futures import ThreadPoolExecutor


def records(action) -> None:
    with ThreadPoolExecutor(max_workers=cpu_count()) as executor:
        executor.submit(Dart().set_version, action=action)
        executor.submit(Docker().set_version, action=action)
        executor.submit(Dotnet().set_version, action=action)
        executor.submit(Elixir().set_version, action=action)
        executor.submit(Golang().set_version, action=action)
        executor.submit(Gulp().set_version, action=action)
        executor.submit(Java().set_version, action=action)
        executor.submit(Julia().set_version, action=action)
        executor.submit(NodeJs().set_version, action=action)
        executor.submit(Php().set_version, action=action)
        executor.submit(Ruby().set_version, action=action)
        executor.submit(Rust().set_version, action=action)
        executor.submit(Scala().set_version, action=action)
        executor.submit(Perl().set_version, action=action)
        executor.submit(CMake().set_version, action=action)
        executor.submit(Crystal().set_version, action=action)
        executor.submit(Deno().set_version, action=action)
        executor.submit(Erlang().set_version, action=action)
        executor.submit(Helm().set_version, action=action)
        executor.submit(Kotlin().set_version, action=action)
        executor.submit(Nim().set_version, action=action)
        executor.submit(Ocaml().set_version, action=action)
        executor.submit(Vagrant().set_version, action=action)
        executor.submit(Zig().set_version, action=action)

# Thread(target=Dart().set_version, kwargs={"action": action}).start()
# Thread(target=Docker().set_version, kwargs={"action": action}).start()
# Thread(target=Dotnet().set_version, kwargs={"action": action}).start()
# Thread(target=Elixir().set_version, kwargs={"action": action}).start()
# Thread(target=Golang().set_version, kwargs={"action": action}).start()
# Thread(target=Gulp().set_version, kwargs={"action": action}).start()
# Thread(target=Java().set_version, kwargs={"action": action}).start()
# Thread(target=Julia().set_version, kwargs={"action": action}).start()
# Thread(target=NodeJs().set_version, kwargs={"action": action}).start()
# Thread(target=Php().set_version, kwargs={"action": action}).start()
# Thread(target=Ruby().set_version, kwargs={"action": action}).start()
# Thread(target=Rust().set_version, kwargs={"action": action}).start()
# Thread(target=Scala().set_version, kwargs={"action": action}).start()
# Thread(target=Perl().set_version, kwargs={"action": action}).start()
# Thread(target=CMake().set_version, kwargs={"action": action}).start()
# Thread(target=Crystal().set_version, kwargs={"action": action}).start()
# Thread(target=Deno().set_version, kwargs={"action": action}).start()
# Thread(target=Erlang().set_version, kwargs={"action": action}).start()
# Thread(target=Helm().set_version, kwargs={"action": action}).start()
# Thread(target=Kotlin().set_version, kwargs={"action": action}).start()
# Thread(target=Nim().set_version, kwargs={"action": action}).start()
# Thread(target=Ocaml().set_version, kwargs={"action": action}).start()
# Thread(target=Vagrant().set_version, kwargs={"action": action}).start()
# Thread(target=Zig().set_version, kwargs={"action": action}).start()



from os.path import join
from snakypy.helpers.console import loading
from snakypy.zshpower.commands.lib.handle import records
from snakypy.zshpower import __version__
from snakypy.zshpower.config.cron import cron_content, sync_content
from snakypy.zshpower.config.base import Base
from snakypy.helpers.ansi import FG, NONE
from snakypy.zshpower.database.dao import DAO
from snakypy.helpers import printer
from snakypy.zshpower.config import package
from snakypy.helpers.path import create as snakypy_path_create
from snakypy.helpers.files import create_file
from snakypy.helpers.catches import tools_requirements
from snakypy.zshpower.utils.catch import get_line_source
from snakypy.zshpower.config.config import content as config_content
from snakypy.zshpower.config.zshrc import content as zshrc_content
from snakypy.zshpower.config.set_zshpower import content as set_zshpower_content
from snakypy.zshpower.utils.process import change_shell, reload_zsh
from snakypy.zshpower.utils.shift import (
    create_config,
    omz_install,
    omz_install_plugins,
    install_fonts,
    create_zshrc,
    change_theme_in_zshrc,
    add_plugins_zshrc,
    cron_task,
    remove_versions_garbage,
)

instruction_not_omz = f"""{FG().YELLOW}
********************** WARNING **********************
Add the following code to the {FG().MAGENTA}$HOME/.zshrc{NONE} {FG().YELLOW}file:

CODE: {FG().CYAN}source $HOME/.zshpower/{__version__}/init.sh {NONE}
{FG().YELLOW}*****************************************************{NONE}
"""


class InitCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self, arguments, *, reload=False, message=False) -> None:
        # printer("Please wait ... assigning settings ...", foreground=FG.WARNING)
        tools_requirements("zsh", "vim", "git", "cut", "grep", "whoami", "pwd")
        # create_zshrc_not_exists(
        #     f". $HOME/.{package.info['pkg_name']}/{__version__}/init.sh", self.zsh_rc
        # )
        snakypy_path_create(self.data_root)
        create_config(config_content, self.config_file)
        create_file(set_zshpower_content, self.init_file, force=True)
        # Create table if not exists
        DAO().create_table(self.tbl_main)
        # Insert registers
        records("insert")
        loading(
            set_time=0.040,
            bar=False,
            header="ZSHPower is creating the database. Wait a moment...",
            foreground=FG().QUESTION,
        )

        if arguments["--omz"]:
            omz_install(self.omz_root)
            omz_install_plugins(self.omz_root, self.plugins)
            create_zshrc(zshrc_content, self.zsh_rc)
            change_theme_in_zshrc(self.zsh_rc, f"{package.info['pkg_name']}")
            add_plugins_zshrc(self.zsh_rc)
            create_file(set_zshpower_content, self.theme_file, force=True)

        install_fonts(self.HOME)
        change_shell()
        remove_versions_garbage(join(self.HOME, f".{package.info['pkg_name']}"))

        try:
            cron_task(sync_content, self.sync_path, cron_content, self.cron_path)
            printer("Done!", foreground=FG().FINISH) if message else None

            if not arguments["--omz"] and not get_line_source(self.zsh_rc):
                printer(instruction_not_omz, foreground=FG().YELLOW)

            if reload:
                reload_zsh()
        except KeyboardInterrupt:
            printer("Canceled by user", foreground=FG().WARNING)
            printer("Done!", foreground=FG().FINISH) if message else None

            if not arguments["--omz"] and not get_line_source(self.zsh_rc):
                printer(instruction_not_omz, foreground=FG().YELLOW)

            if reload:
                reload_zsh()



from os.path import join, exists
from snakypy.zshpower.config import package


def checking_init(home) -> bool:
    """Function that ends commands that depend on the created repository, but
    the repository was not created."""

    if not exists(join(home, f".{package.info['pkg_name']}")):
        raise FileNotFoundError(
            f'Command "{package.info["pkg_name"]} init" has not been started.'
            "Aborted",
        )
    return True
